@@ -25,10 +25,13 @@ import io
 import shutil
 
 from firewall.fw_types import *
-from firewall.functions import splitArgs, joinArgs
+from firewall.functions import splitArgs, joinArgs, u2b_if_py2
 from firewall.errors import *
 from firewall.core.io.io_object import *
 from firewall.core.logger import log
+from firewall.core import ipXtables
+from firewall.core import ebtables
+
 
 class direct_ContentHandler(IO_Object_ContentHandler):
     def __init__(self, item):
@@ -51,7 +54,8 @@ class direct_ContentHandler(IO_Object_ContentHandler):
             ipv = attrs["ipv"]
             table = attrs["table"]
             chain = attrs["chain"]
-            self.item.add_chain(ipv, table, chain)
+            self.item.add_chain(u2b_if_py2(ipv), u2b_if_py2(table),
+                                u2b_if_py2(chain))
 
         elif name == "rule":
             if not self.direct:
@@ -69,14 +73,15 @@ class direct_ContentHandler(IO_Object_ContentHandler):
                 log.error("Parse Error: %s is not a valid priority" % 
                           attrs["priority"])
                 return
-            self._rule = [ipv, table, chain, priority]
+            self._rule = [ u2b_if_py2(ipv), u2b_if_py2(table),
+                           u2b_if_py2(chain), priority ]
 
         elif name == "passthrough":
             if not self.direct:
                 log.error("Parse Error: command outside of direct")
                 return
             ipv = attrs["ipv"]
-            self._passthrough = [ipv]
+            self._passthrough = [ u2b_if_py2(ipv) ]
 
         else:
             log.error('Unknown XML element %s' % name)
@@ -88,7 +93,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
         if name == "rule":
             if self._element:
                 # add arguments
-                self._rule.append(splitArgs(self._element))
+                self._rule.append([ u2b_if_py2(x) for x in splitArgs(self._element) ])
                 self.item.add_rule(*self._rule)
             else:
                 log.error("Error: rule does not have any arguments, ignoring.")
@@ -96,7 +101,7 @@ class direct_ContentHandler(IO_Object_ContentHandler):
         elif name == "passthrough":
             if self._element:
                 # add arguments
-                self._passthrough.append(splitArgs(self._element))
+                self._passthrough.append([ u2b_if_py2(x) for x in splitArgs(self._element) ])
                 self.item.add_passthrough(*self._passthrough)
             else:
                 log.error("Error: passthrough does not have any arguments, " +
@@ -140,17 +145,17 @@ class Direct(IO_Object):
         x = [ ]
         for key in self.chains:
             for chain in self.chains[key]:
-                x.append(list(key) + list([chain]))
+                x.append(tuple(list(key) + list([chain])))
         ret.append(x)
         x = [ ]
         for key in self.rules:
             for rule in self.rules[key]:
-                x.append(list(key) + list(rule))
+                x.append(tuple((key[0], key[1], key[2], rule[0], list(rule[1]))))
         ret.append(x)
         x = [ ]
         for key in self.passthroughs:
             for rule in self.passthroughs[key]:
-                x.append((key, rule))
+                x.append(tuple((key, list(rule))))
         ret.append(x)
         return tuple(ret)
 
@@ -188,9 +193,25 @@ class Direct(IO_Object):
             for args in self.passthroughs[key]:
                 print ("    ('%s')" % ("','".join(args)))
 
+    def _check_ipv(self, ipv):
+        ipvs = ['ipv4', 'ipv6', 'eb']
+        if ipv not in ipvs:
+            raise FirewallError(INVALID_IPV,
+                                "'%s' not in '%s'" % (ipv, ipvs))
+
+    def _check_ipv_table(self, ipv, table):
+        self._check_ipv(ipv)
+
+        tables = ipXtables.BUILT_IN_CHAINS.keys() if ipv in ['ipv4', 'ipv6'] \
+                                         else ebtables.BUILT_IN_CHAINS.keys()
+        if table not in tables:
+            raise FirewallError(INVALID_TABLE,
+                                "'%s' not in '%s'" % (table, tables))
+
     # chains
 
     def add_chain(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table)
         if key not in self.chains:
             self.chains[key] = [ ]
@@ -202,6 +223,7 @@ class Direct(IO_Object):
                         + "already in list, ignoring")
 
     def remove_chain(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table)
         if key in self.chains and chain in self.chains[key]:
             self.chains[key].remove(chain)
@@ -213,10 +235,12 @@ class Direct(IO_Object):
                 (chain, table, ipv))
 
     def query_chain(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table)
         return (key in self.chains and chain in self.chains[key])
 
     def get_chains(self, ipv, table):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table)
         if key in self.chains:
             return self.chains[key]
@@ -230,6 +254,7 @@ class Direct(IO_Object):
     # rules
 
     def add_rule(self, ipv, table, chain, priority, args):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table, chain)
         if key not in self.rules:
             self.rules[key] = LastUpdatedOrderedDict()
@@ -243,6 +268,7 @@ class Direct(IO_Object):
                         + "already in list, ignoring")
 
     def remove_rule(self, ipv, table, chain, priority, args):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table, chain)
         value = (priority, tuple(args))
         if key in self.rules and value in self.rules[key]:
@@ -255,6 +281,7 @@ class Direct(IO_Object):
                 "with ipv '%s' and priority %d not in list" % (ipv, priority))
 
     def remove_rules(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table, chain)
         if key in self.rules:
             for value in self.rules[key].keys():
@@ -263,11 +290,13 @@ class Direct(IO_Object):
                 del self.rules[key]
 
     def query_rule(self, ipv, table, chain, priority, args):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table, chain)
         value = (priority, tuple(args))
         return (key in self.rules and value in self.rules[key])
 
     def get_rules(self, ipv, table, chain):
+        self._check_ipv_table(ipv, table)
         key = (ipv, table, chain)
         if key in self.rules:
             return self.rules[key]
@@ -281,6 +310,7 @@ class Direct(IO_Object):
 #    # passthrough
 #
     def add_passthrough(self, ipv, args):
+        self._check_ipv(ipv)
         if ipv not in self.passthroughs:
             self.passthroughs[ipv] = [ ]
         if args not in self.passthroughs[ipv]:
@@ -291,6 +321,7 @@ class Direct(IO_Object):
                         + "already in list, ignoring")
 
     def remove_passthrough(self, ipv, args):
+        self._check_ipv(ipv)
         if ipv in self.passthroughs and args in self.passthroughs[ipv]:
             self.passthroughs[ipv].remove(args)
             if len(self.passthroughs[ipv]) == 0:
@@ -300,9 +331,11 @@ class Direct(IO_Object):
                 ("',".join(args), ipv) + "not in list"
 
     def query_passthrough(self, ipv, args):
+        self._check_ipv(ipv)
         return (ipv in self.passthroughs and args in self.passthroughs[ipv])
 
     def get_passthroughs(self, ipv):
+        self._check_ipv(ipv)
         if ipv in self.passthroughs:
             return self.passthroughs[ipv]
         else:
