@@ -108,7 +108,8 @@ DEFAULT_RULES["filter"] = [
     "-I INPUT 4 -j INPUT_ZONES_SOURCE",
     "-I INPUT 5 -j INPUT_ZONES",
     "-I INPUT 6 -p %%ICMP%% -j ACCEPT",
-    "-I INPUT 7 -j %%REJECT%%",
+    "-I INPUT 7 -m conntrack --ctstate INVALID -j DROP",
+    "-I INPUT 8 -j %%REJECT%%",
 
     "-N FORWARD_direct",
     "-N FORWARD_IN_ZONES_SOURCE",
@@ -124,7 +125,8 @@ DEFAULT_RULES["filter"] = [
     "-I FORWARD 6 -j FORWARD_OUT_ZONES_SOURCE",
     "-I FORWARD 7 -j FORWARD_OUT_ZONES",
     "-I FORWARD 8 -p %%ICMP%% -j ACCEPT",
-    "-I FORWARD 9 -j %%REJECT%%",
+    "-I FORWARD 9 -m conntrack --ctstate INVALID -j DROP",
+    "-I FORWARD 10 -j %%REJECT%%",
 
     "-N OUTPUT_direct",
 
@@ -141,10 +143,14 @@ class ip4tables:
 
     def __init__(self):
         self._command = COMMAND[self.ipv]
+        self.wait_option = self._detect_wait_option()
 
     def __run(self, args):
         # convert to string list
-        _args = ["%s" % item for item in args]
+        if self.wait_option:
+            _args = [self.wait_option] + ["%s" % item for item in args]
+        else:
+            _args = ["%s" % item for item in args]
         log.debug2("%s: %s %s", self.__class__, self._command, " ".join(_args))
         (status, ret) = runProg(self._command, _args)
         if status != 0:
@@ -186,6 +192,18 @@ class ip4tables:
 
         return tables
 
+    def _detect_wait_option(self):
+        wait_option = ""
+        (status, ret) = runProg(self._command, ["-w", "-L"])  # since iptables-1.4.20
+        if status == 0:
+            wait_option = "-w"  # wait for xtables lock
+            (status, ret) = runProg(self._command, ["-w2", "-L"])  # since iptables > 1.4.21
+            if status == 0:
+                wait_option = "-w2"  # wait max 2 seconds
+            log.debug2("%s: %s will be using %s option.", self.__class__, self._command, wait_option)
+
+        return wait_option
+
     def flush(self):
         tables = self.used_tables()
         for table in tables:
